* Melanie Krause, www.melanie-krause.de
* Sebastian Kripfganz, www.kripfganz.de

* Replication code for:
// Krause, M., and S. Kripfganz (2025). Regional dependencies and local spillovers: Insights from commuter flows. Journal of Regional Science.
// Article DOI: 10.1111/jors.12752

// Note: This code provides the raw estimates for the respective tables in the main paper and supplementary appendix. It does not produce formatted tables. Figures are produced as they appear in the paper.
// The code should be executed at one go; executing only parts of it will likely fail.

*==================================================*
version 16.1		// minimum required Stata version
clear all

**** required packages ****
ssc install ineqdeco
net install sg162, from(http://www.stata.com/stb/stb60)
ssc install sppack
ssc install spmap
net install spxtdpdbc, from(http://www.kripfganz.de/stata)
net install xtdpdbc, from(http://www.kripfganz.de/stata)
net install xtdpdgmm, from(http://www.kripfganz.de/stata)

**** spatial weight matrices (creates new files) ****
foreach wmat in comm_pop_spec cont_spec dist_spec {
	if "`wmat'" == "comm_pop_spec" {
		forv y = 2002/2017 {
			qui import delim data/wmat_`wmat'_`y'.txt, delim(whitespace) varn(1) clear
			save data/wmat_`wmat'_`y'.dta, replace
		}
	}
	else {
		qui import delim data/wmat_`wmat'.txt, delim(whitespace) varn(1) clear
		save data/wmat_`wmat'.dta, replace
	}
}

**** data set ****
use data/data_final
describe
notes

*==================================================*
**** descriptive statistics ****

*--------------------------------------------------*
*** Table A.2 (supplementary appendix) ***
** Table A.2: summary statistics for raw data **
sum gdp_cap gva* investment_proc pop_tot area cpi worker commuter_in commuter_out educ1 educ2
sum prices_2009 if year == 2009

*--------------------------------------------------*
*** Figures 1 and 2 (main paper) ***
preserve

** Figure 1: shape, location, and geographic centroids of four selected German counties **
qui gen label = county_name if inlist(id, 118, 120, 124, 126)
qui replace label = "Frankfurt" if id == 118
qui replace label = "Wiesbaden" if id == 120
qui replace label = "Hochtaunus" if id == 124
qui replace label = "Main-Taunus" if id == 126
spmap using data/vg2500_krs_coord if year == 2002 & inlist(id, 118, 120, 124, 126), id(id) point(xcoord(x_center) ycoord(y_center) select(keep if inlist(id, 118, 120, 124, 126))) label(label(label) xcoord(x_center) ycoord(y_center) position(12) gap(*4) size(medium)) ysize(*0.75) xsize(*0.75) name(map_example, replace)

** Figure 2: spatial distribution of the per capita flow of commuters from and to the 401 German counties in 2002 **
format commuter_out_cap %4.3f
spmap commuter_out_cap using data/vg2500_krs_coord if year == 2002, id(id) fc(Blues) cln(4) legend(position(11)) name(commuter_out_cap, replace)
format commuter_in_cap %4.3f
spmap commuter_in_cap using data/vg2500_krs_coord if year == 2002, id(id) fc(Blues) cln(4) legend(position(11)) name(commuter_in_cap, replace)

restore

*--------------------------------------------------*
*** Table A.3 (supplementary appendix) ***
** Table A.3: summary statistics for real GDP per capita **
bys year: sum rgdp_cap, d
xtset id year
forv y = 2002/2017 {
	ineqdeco rgdp_cap if year == `y'		// Gini coefficient
}

** Spearman rank correlation (page 19 of the main paper) **
preserve
keep id year rgdp_cap
reshape wide rgdp_cap, i(id) j(year)
spearman rgdp_cap2002 rgdp_cap2017
forv y = 2003/2017 {
	spearman rgdp_cap`=`y'-1' rgdp_cap`y'
}
restore

*--------------------------------------------------*
*** Figure 3 (main paper) and Figure A.1 (supplementary appendix) ***
preserve
collapse (first) rgdp_cap (mean) ggdp_cap, by(id)

** Figure 3: spatial distribution of real GDP per capita of the 401 German counties in 2002 and its average annual growth rate from 2002 to 2017 **
format rgdp_cap %7.0f
format ggdp_cap %4.3f
spmap rgdp_cap using data/vg2500_krs_coord, id(id) fc(Blues) cln(4) legend(position(11)) name(rgdp_cap, replace)
spmap ggdp_cap using data/vg2500_krs_coord, id(id) fc(Blues) cln(4) legend(position(11)) name(ggdp_cap, replace)

** Figure A.1: distribution of real GDP per capita of the 401 German counties in 2002 and its average annual growth rate from 2002 to 2017 **
quantile rgdp_cap, rlopts(lc(black) lp(dot)) recast(line) xtitle("") xlabel(, grid format(%3.2f)) ytitle("") ylabel(16138 24975 29801 36490 105143, angle(horizontal) format(%7.0fc)) scheme(s2mono) graphregion(fcolor(white) lcolor(white)) name(rgdp_cap_q, replace)
twoway (scatter ggdp_cap rgdp_cap, msize(tiny)) (lfit ggdp_cap rgdp_cap), xtitle("initial real GDP per capita") xlabel(, format(%7.0fc)) ytitle("average real GDP per capita growth") ylabel(, angle(horizontal) format(%3.2f)) scheme(s2mono) graphregion(fcolor(white) lcolor(white)) legend(off) name(ggdp_rgdp, replace)

restore

*--------------------------------------------------*
*** Table A.4 (supplementary appendix) ***
** Table A.4: summary statistics for regression variables **
sum lrgdp_cap commuter_out_cap commuter_in_cap
sum commuter_out_cap commuter_in_cap if year == 2002
sum lrinv_cap gpop L.educ1_worker L.educ2_worker if year > 2002

*--------------------------------------------------*
*** Table A.5 (supplementary appendix) ***
** Table A.5: summary statistics for counterfactual treatmeant variables **
sum gdp_cap sgva* pop_area if year == 2002
foreach var of varlist gdp_cap sgva* pop_area {
	sum `var' if year == 2002, d
}

*--------------------------------------------------*
*** Table A.6 (supplementary appendix) and Figure 4 (main paper) ***
** Table A.6: Moran's I measure of global spatial autocorrelation in real GDP per capita **
spatwmat using data/wmat_cont_spec.dta, name(wmat_cont_spec)
spatwmat using data/wmat_dist_spec.dta, name(wmat_dist_spec)
forv y = 2002/2017 {
	preserve
	keep if year == `y'
	spatwmat using data/wmat_comm_pop_spec_`y'.dta, name(wmat_comm_pop_spec_`y')
	spatwmat using data/wmat_comm_pop_spec_`y'.dta, name(wmat_comm_pop_spec_`y')
	spatgsa rgdp_cap, weights(wmat_comm_pop_spec_`y') moran twotail
	loc z_comm_`y' = r(Moran)[1, "z"]
	spatgsa rgdp_cap, weights(wmat_cont_spec) moran twotail
	loc z_cont_`y' = r(Moran)[1, "z"]
	spatgsa rgdp_cap, weights(wmat_dist_spec) moran twotail
	loc z_dist_`y' = r(Moran)[1, "z"]
	restore
}

** Figure 4: Moran's I global spatial autocorrelation over time **
qui gen z_comm = .
label var z_comm "commuter {bf:W}{sub:N}"
qui gen z_cont = .
label var z_cont "contiguity {bf:W}{sub:N}"
qui gen z_dist = .
label var z_dist "inverse-distance {bf:W}{sub:N}"
forv y = 2002/2017 {
	qui replace z_comm = `z_comm_`y'' if year == `y' & id == 1
	qui replace z_cont = `z_cont_`y'' if year == `y' & id == 1
	qui replace z_dist = `z_dist_`y'' if year == `y' & id == 1
}
twoway (tsline z_comm) (tsline z_cont) (tsline z_dist), ytitle("z", orientation(horizontal)) ylabel(-4 -3 -1 0 1 3 4, angle(horizontal) nogrid) ymlabel(`=invnorm(0.005)' `=invnorm(0.025)' `=invnorm(0.05)' `=invnorm(0.95)' `=invnorm(0.975)' `=invnorm(0.995)', format(%3.2f) angle(horizontal) grid) ymtick(0, grid) xlabel(2002(1)2017) legend(rows(1) colgap(50) region(lcolor(white))) scheme(s2mono) graphregion(fcolor(white) lcolor(white)) ysize(2) name(moran, replace)
drop z_comm z_cont z_dist

*--------------------------------------------------*
*** Table A.7 (supplementary appendix) and Figure 5 (main paper) ***
** Table A.7: Moran's I measure of local spatial autocorrelation in real GDP per capita **
** Figure 5: Moran's I local spatial autocorrelation in 2002 **
preserve
keep if year == 2002
spatlsa rgdp_cap, weights(wmat_comm_pop_spec_2002) moran twotail id(id)
mat M = r(Moran)[., "z"]
svmat M, names(col)
spmap z using data/vg2500_krs_coord, id(id) fc(BuRd) clmethod(custom) clbreaks(-20 -2.58 -1.96 -1.64 0 1.64 1.96 2.58 20) legend(position(11)) legend(label(2 "(-{&infin},-2.58]") label(9 "(2.58,{&infin})")) name(moran_comm, replace)
drop z
spatlsa rgdp_cap, weights(wmat_cont_spec) moran twotail id(id)
mat M = r(Moran)[., "z"]
svmat M, names(col)
spmap z using data/vg2500_krs_coord, id(id) fc(BuRd) clmethod(custom) clbreaks(-20 -2.58 -1.96 -1.64 0 1.64 1.96 2.58 20) legend(position(11)) legend(label(2 "(-{&infin},-2.58]") label(9 "(2.58,{&infin})")) name(moran_cont, replace)
drop z
spatlsa rgdp_cap, weights(wmat_dist_spec) moran twotail id(id)
mat M = r(Moran)[., "z"]
svmat M, names(col)
spmap z using data/vg2500_krs_coord, id(id) fc(BuRd) clmethod(custom) clbreaks(-20 -2.58 -1.96 -1.64 0 1.64 1.96 2.58 20) legend(position(11)) legend(label(2 "(-{&infin},-2.58]") label(9 "(2.58,{&infin})")) name(moran_dist, replace)
drop z
restore

*==================================================*
**** regression results ****
* regression variables *
xi, pre(_) i.year
by id: gen crisis_manu = sgva_manu[1] * (year >= 2008)
by id: gen crisis_cons = sgva_cons[1] * (year >= 2008)
by id: gen crisis_fin = sgva_fin[1] * (year >= 2008)
by id: gen crisis_trade = sgva_trade[1] * (year >= 2008)
loc depvar			"lrgdp_cap"
loc indepvars_endo	"lrinv_cap"
loc indepvars_pre	"gpop"
loc indepvars_exo	"L.educ1_worker L.educ2_worker"
loc indepvars		"`indepvars_endo' `indepvars_pre' `indepvars_exo'"
loc tdummies		"_year_2004-_year_2017"
loc cdummies		"crisis*"
loc wmats			"comm_pop_spec_2002 cont_spec dist_spec"
* spatial weight matrices *
foreach wmat of loc wmats {
	spmat import wmat_`wmat' using data/wmat_`wmat'.txt, noid replace
	spmat getmatrix wmat_`wmat' W_`wmat' 
}

*--------------------------------------------------*
*** Table 1 (main paper) ***
** Table 1: bias‐corrected QML estimation of time–space dynamic panel models **
forv j = 1 / `: word count `wmats'' {
	loc wmat : word `j' of `wmats'
	loc regressors "`indepvars'"
	forv e = 1 / 3 {
		if `e' == 2 {
			loc regressors "`regressors' `tdummies'"
		}
		else if `e' == 3 {
			loc regressors "`regressors' `cdummies'"
		}
		spxtdpdbc `depvar' `regressors', spmat(W_`wmat', m)
		di "LL: " as res e(ll)		// value of the log-likelihood function
		if `e' == 3 {
			mata: V`j' = st_matrix("e(V)")
			mata: s = (1 \ rows(V`j')-1 \ rows(V`j'))
			mata: V`j' = V`j'[s, s']
			mata: SRSM`j' = luinv(I(401) - `=_b[W:`depvar']' * W_`wmat')		// short-run spatial multiplier matrix
			mata: SRSM`j'_2W = SRSM`j' * W_`wmat' * SRSM`j'
			mata: LRSM`j' = luinv((1 - `=_b[L.`depvar']') * I(401) - (`=_b[W:`depvar']' + `=_b[W:L.`depvar']') * W_`wmat')		// long-run spatial multiplier matrix
			mata: LRSM`j'_2 = LRSM`j' * LRSM`j'
			mata: LRSM`j'_2W = LRSM`j' * W_`wmat' * LRSM`j'
		}
	}
}

*--------------------------------------------------*
*** Table B.1 (supplementary appendix) ***
** Table B.1: bias-corrected QML estimation with commuter flows from different years **
foreach wmat in comm_pop_spec_2017 comm_pop_avg_spec {
	spmat import wmat_`wmat' using data/wmat_`wmat'.txt, noid replace
	spmat getmatrix wmat_`wmat' W_`wmat' 
	loc regressors "`indepvars'"
	forv e = 1 / 3 {
		if `e' == 2 {
			loc regressors "`regressors' `tdummies'"
		}
		else if `e' == 3 {
			loc regressors "`regressors' `cdummies'"
		}
		spxtdpdbc `depvar' `regressors', spmat(W_`wmat', m)
	}
}

*--------------------------------------------------*
*** Table B.2 (supplementary appendix) ***
** Table B.2: alternative estimators for the dynamic panel models **
* fixed-effects (FE) estimator *
xtreg `depvar' L.`depvar' `indepvars' `tdummies' `cdummies', fe vce(r)
* bias-corrected fixed-effects (FE-BC) estimator *
xtdpdbc `depvar' `indepvars' `tdummies' `cdummies', fe vce(r)
estat serial		// Arellano-Bond serial correlation test
* generalized method of moments (GMM) estimator *
xtdpdgmm `depvar' L.`depvar' `indepvars' `tdummies' `cdummies', nl(noserial) gmm(L.`depvar', lag(0 4) m(fodev) collapse) gmm(`indepvars_endo', m(fodev) lag(1 5) collapse) gmm(`indepvars_pre', m(fodev) lag(0 4) collapse) iv(`indepvars_exo' `tdummies' `cdummies', m(mdev)) twostep vce(robust)
estat serial		// Arellano-Bond serial correlation test
estat overid		// Hansen overidentification test
foreach wmat of loc wmats {
	xtdpdgmm `depvar' L.`depvar' `indepvars' `tdummies' `cdummies' W`depvar'_`wmat' L.W`depvar'_`wmat', nl(noserial) gmm(L.`depvar' L.W`depvar'_`wmat', lag(0 4) m(fodev) collapse) gmm(`indepvars_endo', m(fodev) lag(1 5) collapse) gmm(`indepvars_pre', m(fodev) lag(0 4) collapse) iv(`indepvars_exo' `tdummies' `cdummies', m(mdev)) twostep vce(robust)
	estat serial		// Arellano-Bond serial correlation test
	estat overid		// Hansen overidentification test
}

*--------------------------------------------------*
*** Table 2 (main paper) and Figure 6 (main paper) ***
** Table 2: average short-run and long-run spatial multipliers **
** Figure 6: indirect long-run spill-in multipliers **
keep if year == 2002
forv j = 1 / `: word count `wmats'' {
	* average short-run direct spatial multiplier *
	qui gen srme_d_`j' = .
	mata: st_store((1, 401), "srme_d_`j'", diagonal(SRSM`j'))
	sum srme_d_`j'
	mata: sqrt(V`j'[2, 2]) * trace(SRSM`j'_2W) / 401		// Delta-method standard error
	* average short-run total spatial multiplier *
	qui gen srme_t_`j' = .
	mata: st_store((1, 401), "srme_t_`j'", rowsum(SRSM`j'))
	sum srme_t_`j'
	mata: sqrt(V`j'[2, 2]) * sum(SRSM`j'_2W) / 401		// Delta-method standard error
	* average short-run indirect spatial multiplier *
	qui gen srme_i_`j' = srme_t_`j' - srme_d_`j'
	sum srme_i_`j'
	mata: sqrt(V`j'[2, 2]) * (sum(SRSM`j'_2W) - trace(SRSM`j'_2W)) / 401		// Delta-method standard error
}
forv j = 1 / `: word count `wmats'' {
	* average long-run direct spatial multiplier *
	qui gen lrme_d_`j' = .
	mata: st_store((1, 401), "lrme_d_`j'", diagonal(LRSM`j'))
	sum lrme_d_`j'
	mata: g1 = trace(LRSM`j'_2)
	mata: g2 = trace(LRSM`j'_2W)
	mata: sqrt((g1, g2, g2) * V`j' * (g1 \ g2 \ g2)) / 401		// Delta-method standard error
	* average long-run total spatial multiplier *
	qui gen lrme_t_`j' = .
	mata: st_store((1, 401), "lrme_t_`j'", rowsum(LRSM`j'))
	sum lrme_t_`j'
	mata: g1 = sum(LRSM`j'_2)
	mata: g2 = sum(LRSM`j'_2W)
	mata: sqrt((g1, g2, g2) * V`j' * (g1 \ g2 \ g2)) / 401		// Delta-method standard error
	* average long-run indirect spatial multiplier *
	qui gen lrme_i_`j' = lrme_t_`j' - lrme_d_`j'
	sum lrme_i_`j'
	mata: g1 = sum(LRSM`j'_2) - trace(LRSM`j'_2)
	mata: g2 = sum(LRSM`j'_2W) - trace(LRSM`j'_2W)
	mata: sqrt((g1, g2, g2) * V`j' * (g1 \ g2 \ g2)) / 401		// Delta-method standard error
	format lrme_i_`j' %4.3f
	spmap lrme_i_`j' using "data/vg2500_krs_coord", id(id) fc(Blues) legend(position(11)) name(lrme_i_`j', replace)
}

*--------------------------------------------------*
*** Table 3 (main paper), Figures 7 to 9 (main paper), Figures B.1 to B.4 (supplementary appendix), and Table B.3 (supplementary appendix) ***
** Table 3: average counterfactual long-run spatial multipliers **
** Figure 7: counterfactual long-run spatial multipliers for treatment of financial centers **
** Figure B.1: counterfactual long-run spatial multipliers for treatment of industrial centers **
** Figure B.2: counterfactual long-run spatial multipliers for treatment of agricultural centers **
** Figure 8: counterfactual long-run spatial multipliers for treatment of rich counties **
** Figure 9: counterfactual long-run spatial multipliers for treatment of poor counties **
** Figure B.3: counterfactual long-run spatial multipliers for treatment of urban counties **
** Figure B.4: counterfactual long-run spatial multipliers for treatment of rural counties **
** Table B.3: total counterfactual long-run returns **
loc origins			"sgva_fin sgva_manu sgva_agri gdp_cap gdp_cap pop_area pop_area"
loc cutoff_up		"1 1 1 1 0 1 0"
gen shock = ln(10000 / rgdp + 1)
loc o = 0
foreach origin of loc origins {
    loc ++o
	if `: word `o' of `cutoff_up'' {
		_pctile `origin', p(95)
		gen byte s`o' = (`origin' > r(r1))
	}
	else {
		_pctile `origin', p(5)
		gen byte s`o' = (`origin' < r(r1))
	}
	forv j = 1 / `: word count `wmats'' {
		qui gen lrme`o'_`j' = .
		mata: s = st_data(., "s`o'")
		mata: ns = 1 :- s
		mata: lrme`o'_`j' = LRSM`j' * s
		mata: st_store((1, 401), "lrme`o'_`j'", lrme`o'_`j')
		* average long-run spatial multiplier for the treated *
		sum lrme`o'_`j' if s`o'
		mata: g1 = s' LRSM`j'_2 * s
		mata: g2 = s' LRSM`j'_2W * s
		mata: sqrt((g1, g2, g2) * V`j' * (g1 \ g2 \ g2)) / colsum(s)		// Delta-method standard error
		* average long-run spatial multiplier for the untreated *
		sum lrme`o'_`j' if !s`o'
		mata: g1 = ns' LRSM`j'_2 * s
		mata: g2 = ns' LRSM`j'_2W * s
		mata: sqrt((g1, g2, g2) * V`j' * (g1 \ g2 \ g2)) / (rows(s) - colsum(s))		// Delta-method standard error
		sort lrme`o'_`j'
		gen int cid`o'_`j' = _n
		sort id year
		qui gen lrmeu`o'_`j' = lrme`o'_`j' if !s`o'
		spmap lrmeu`o'_`j' using "data/vg2500_krs_coord", id(id) fc(Blues) legend(position(11)) ndfcolor("153 0 13") ndlabel("treated") clmethod(custom) clbreaks(0 0.125 0.25 1 4) legend(label(5 "(1,{&infin})")) name(lrme`o'_`j', replace)
		* total long-run return on the investment for the treated and untreated *
		qui gen lre`o'_`j' = .
		mata: eps = s :* st_data(., "shock")
		mata: lre`o'_`j' = LRSM`j' * eps
		mata: st_store((1, 401), "lre`o'_`j'", lre`o'_`j')
		gen alre`o'_`j' = rgdp * (exp(lre`o'_`j') - 1) / 10000
		sum alre`o'_`j' if s`o'		// treated
		sum alre`o'_`j' if !s`o'
		di 381 * r(mean) / 20		// untreated
		sum alre`o'_`j'
		di 401 * r(mean) / 20		// all
	}
}
